@@ -1,4 +1,4 @@
-#!/usr/bin/env python
+#!/usr/bin/env python3
 # coding=utf-8
 """
 Test specific elements API from svg xml lxml custom classes.
@@ -40,7 +40,8 @@ from inkex import paths
 from inkex.colors import Color
 from inkex.paths import Move, Line
 from inkex.utils import FragmentError
-from inkex.units import parse_unit
+from inkex.units import parse_unit, convert_unit
+from inkex.transforms import BoundingBox
 
 from .test_inkex_elements_base import SvgTestCase
 from inkex.tester.svg import svg
@@ -483,16 +484,31 @@ class NamedViewTest(ElementTestCase):
 
     def test_guides(self):
         """Create a guide and see a list of them"""
-        self.svg.namedview.add(Guide().move_to(0, 0, 0))
-        self.svg.namedview.add(Guide().move_to(0, 0, "90"))
+        self.svg.namedview.add_guide(0, False, 0)
+        self.svg.namedview.add_guide(0, False, "90")
         self.assertEqual(len(self.svg.namedview.get_guides()), 2)
+        self.svg.namedview.add_guide((1, 1), (1, 0))
+        guides = self.svg.namedview.get_guides()
+        self.assertAlmostEqual(
+            guides[2].raw_position.y, self.svg.viewport_height - 1, 2
+        )
+        self.assertAlmostEqual(guides[2].raw_position.x, 1, 2)
+        self.assertAlmostEqual(guides[2].position.y, 1, 2)
+        self.assertAlmostEqual(guides[2].position.x, 1, 2)
+        # Test angle specifications
+
+    def test_guide_angles(self):
+        g = self.svg.namedview.add_guide((1, 1), 30)
+        self.assertAlmostEqual(g.angle, 30, 4)
+        g = self.svg.namedview.add_guide((1, 1), (2.5, -5 * math.sqrt(3) / 2))
+        self.assertAlmostEqual(g.angle, 30, 4)
 
     def test_guides_coincident(self):
         """Test the detection of coincident guides"""
 
         def coincidence_test(data, result):
-            guide1 = Guide().move_to(*data[0], angle=data[1])
-            guide2 = Guide().move_to(*data[2], angle=data[3])
+            guide1 = self.svg.namedview.add_guide(data[0], orient=data[1])
+            guide2 = self.svg.namedview.add_guide(data[2], orient=data[3])
             self.assertEqual(Guide.guides_coincident(guide1, guide2), result)
 
         # both are good
@@ -513,8 +529,7 @@ class NamedViewTest(ElementTestCase):
 
     def test_pages(self):
         """Create some extra pages and see a list of them"""
-        self.assertEqual(len(self.svg.namedview.get_pages()), 0)
-        self.svg.namedview.add(Page(width="210", height="297", x="0", y="0"))
+        self.assertEqual(len(self.svg.namedview.get_pages()), 1)
         self.svg.namedview.new_page(
             x="220", y="0", width="147.5", height="210", label="TEST"
         )
@@ -524,10 +539,27 @@ class NamedViewTest(ElementTestCase):
             self.svg.namedview.get_pages()[1].get("inkscape:label"), "TEST"
         )
         self.assertEqual(self.svg.namedview.get_pages()[1].attrib["width"], "147.5")
-        self.assertEqual(self.svg.namedview.get_pages()[0].attrib["height"], "297")
+        self.assertAlmostEqual(
+            float(self.svg.namedview.get_pages()[0].attrib["height"]),
+            297 * convert_unit("1mm", "px"),
+        )
         self.assertEqual(self.svg.namedview.get_pages()[1].attrib["x"], "220")
         self.assertEqual(self.svg.namedview.get_pages()[1].attrib["y"], "0")
 
+    def test_get_page_bbox(self):
+        self.assertEqual(
+            self.svg.get_page_bbox(),
+            BoundingBox(
+                (0, 210 * convert_unit("1mm", "px")),
+                (0, 297 * convert_unit("1mm", "px")),
+            ),
+        )
+        self.svg.namedview.add(Page(width="210", height="297", x="0", y="0"))
+        self.svg.namedview.new_page(x="220", y="0", width="147.5", height="210")
+        self.assertEqual(self.svg.get_page_bbox(), BoundingBox((0, 210), (0, 297)))
+        self.assertEqual(self.svg.get_page_bbox(0), BoundingBox((0, 210), (0, 297)))
+        self.assertEqual(self.svg.get_page_bbox(1), BoundingBox((220, 367.5), (0, 210)))
+
     def test_center(self):
         """Test that the center in mm based documents is correctly computed"""
         mmbased = svg(f'width="210mm" viewBox="0 0 210 297"')
@@ -569,8 +601,10 @@ class UseTest(ElementTestCase):
     tag = "use"
 
     def test_path(self):
-        """Use path follows ref"""
+        """Use path follows ref (including refs transform)"""
         self.assertEqual(str(self.elem.path), "M 0 0 L 10 10 Z")
+        self.elem.href.transform = Transform("translate(100, 100)")
+        self.assertEqual(str(self.elem.path), "M 100 100 L 110 110 Z")
 
     def test_empty_ref(self):
         """An empty ref or None ref doesn't cause an error"""
@@ -586,6 +620,47 @@ class UseTest(ElementTestCase):
         elem.set("xlink:href", self.elem.get("xlink:href"))
         self.assertEqual(elem.href.get("id"), "path1")
 
+    def test_href_compat_xlink_getElementsByHref(self):
+        """Test attribute `href` is used according to SVG2 spec
+        while considering `xlink:href` for SVG1.1 compatibility"""
+        # test getElementsByHref `href`
+        elem = self.svg.add(Use())
+        elem.set("href", "#path1")
+        href = self.svg.getElementsByHref("path1")[-1]
+        self.assertEqual(href.TAG, elem.TAG)
+        self.assertEqual(href.get_id(), elem.get_id())
+        # test getElementsByHref `xlink:href`
+        elem = self.svg.add(Use())
+        elem.set("xlink:href", "#path2")
+        href = self.svg.getElementsByHref("path2")[-1]
+        self.assertEqual(href.TAG, elem.TAG)
+        self.assertEqual(href.get_id(), elem.get_id())
+
+    def test_href_compat_xlink_create_read_update(self):
+        """Test attribute `href` is read and updated according to SVG2 spec
+        while creating defaults to `xlink:href` for SVG1.1 compatibility"""
+        # test setter (create) `xlink:href`
+        elem = self.svg.add(Use())
+        elem.href = self.elem.href
+        self.assertEqual(elem.get("xlink:href"), "#path1")
+        self.assertEqual(elem.href.get("id"), "path1")
+        # test setter (update existing) `href` [SVG2]
+        elem = self.svg.add(Use())
+        elem.set("href", "#path2")
+        elem.href = self.elem.href
+        self.assertEqual(elem.get("href"), "#path1")
+        self.assertEqual(elem.href.get("id"), "path1")
+        # test getter (read existing) `href` [SVG2]
+        elem = self.svg.add(Use())
+        elem.set("href", "#path1")
+        self.assertEqual(elem.get("href"), "#path1")
+        self.assertEqual(elem.href.get("id"), "path1")
+        # test getter (read existing) `xlink:href`
+        elem = self.svg.add(Use())
+        elem.set("xlink:href", "#path1")
+        self.assertEqual(elem.get("xlink:href"), "#path1")
+        self.assertEqual(elem.href.get("id"), "path1")
+
     def test_unlink(self):
         """Test use tag unlinking"""
         elem = self.elem.unlink()
