@@ -58,7 +58,9 @@ const char *href_like_attributes[] = {"inkscape:connection-end",
                                       "inkscape:href",
                                       "inkscape:path-effect",
                                       "inkscape:perspectiveID",
+                                      "inkscape:linked-fill",
                                       "inkscape:tiled-clone-of",
+                                      "href",
                                       "xlink:href"};
 #define NUM_HREF_LIKE_ATTRIBUTES (sizeof(href_like_attributes) / sizeof(*href_like_attributes))
 
@@ -121,7 +123,11 @@ fix_ref(IdReference const &idref, SPObject *to_obj, const char *old_id) {
                 old += old_id;
                 size_t posid = value.find(old_id);
                 if (new_uri && posid != Glib::ustring::npos) {
-                    value = value.replace(posid - 1, old.size(), new_uri);
+                    if (!g_strcmp0(idref.attr,"inkscape:linked-fill")) {
+                        value = value.replace(posid, old.size() - 1, to_obj->getId());
+                    } else {
+                        value = value.replace(posid - 1, old.size(), new_uri);
+                    }
                     idref.elem->setAttribute(idref.attr, value.c_str());
                 }
                 g_free(new_uri);
@@ -200,7 +206,7 @@ static void find_references(SPObject *elem, refmap_type &refmap, bool from_clipb
         //}
     }
     if (!std::strcmp(repr_elem->name(), "inkscape:path-effect")) {
-        LivePathEffectObject *lpeobj = dynamic_cast<LivePathEffectObject *>(elem);
+        auto lpeobj = cast<LivePathEffectObject>(elem);
         if (lpeobj) {
             Inkscape::LivePathEffect::Effect *effect = lpeobj->get_lpe();
             if (effect) {
@@ -269,8 +275,15 @@ static void find_references(SPObject *elem, refmap_type &refmap, bool from_clipb
     }
     /* check for xlink:href="#..." and similar */
     for (auto attr : href_like_attributes) {
-        const gchar *val = repr_elem->attribute(attr);
-        if (val && val[0] == '#') {
+        Glib::ustring attfixed = "";
+        if (repr_elem->attribute(attr)) {
+            if (!g_strcmp0(attr,"inkscape:linked-fill")) {
+                attfixed += "#";
+            }
+            attfixed += repr_elem->attribute(attr);
+        }
+        const gchar *val = attfixed.c_str();
+        if (!attfixed.empty() && attfixed[0] == '#') {
             gchar **strarray = g_strsplit(val, ";", 0);
             if (strarray) {
                 unsigned int i = 0;
@@ -376,21 +389,21 @@ static void change_clashing_ids(SPDocument *imported_doc, SPDocument *current_do
         // may have had, the new ID is the old ID followed by a hyphen
         // and one or more digits.
 
-        if (SP_IS_GRADIENT(elem)) {
+        if (is<SPGradient>(elem)) {
             SPObject *cd_obj =  current_doc->getObjectById(id);
 
-            if (cd_obj && SP_IS_GRADIENT(cd_obj)) {
-                SPGradient *cd_gr = SP_GRADIENT(cd_obj);
-                if ( cd_gr->isEquivalent(SP_GRADIENT(elem))) {
+            if (cd_obj && is<SPGradient>(cd_obj)) {
+                auto cd_gr = cast<SPGradient>(cd_obj);
+                if ( cd_gr->isEquivalent(cast<SPGradient>(elem))) {
                     fix_clashing_ids = false;
                  }
              }
         }
 
-        LivePathEffectObject *lpeobj = dynamic_cast<LivePathEffectObject *>(elem);
+        auto lpeobj = cast<LivePathEffectObject>(elem);
         if (lpeobj) {
             SPObject *cd_obj = current_doc->getObjectById(id);
-            LivePathEffectObject *cd_lpeobj = dynamic_cast<LivePathEffectObject *>(cd_obj);
+            auto cd_lpeobj = cast<LivePathEffectObject>(cd_obj);
             if (cd_lpeobj && lpeobj->is_similar(cd_lpeobj)) {
                 fix_clashing_ids = from_clipboard;
             }
@@ -485,8 +498,7 @@ const char valid_id_chars[] = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvw
 /**
  * Modify 'base_name' to create a new ID that is not used in the 'document'
 */
-Glib::ustring generate_unique_id(SPDocument* document, const Glib::ustring& base_name) {
-    const auto NPOS = Glib::ustring::npos;
+Glib::ustring generate_similar_unique_id(SPDocument* document, const Glib::ustring& base_name) {
     // replace illegal chars in base_name
     auto id = base_name;
     if (id.empty()) {
@@ -494,7 +506,7 @@ Glib::ustring generate_unique_id(SPDocument* document, const Glib::ustring& base
     }
     else {
         for (auto pos = id.find_first_not_of(valid_id_chars);
-                  pos != NPOS;
+                  pos != Glib::ustring::npos;
                   pos = id.find_first_not_of(valid_id_chars, pos)) {
             id.replace(pos, 1, "_");
         }
