@@ -28,6 +28,7 @@
 
 #include "libnrtype/font-instance.h"
 #include "libnrtype/font-lister.h"
+#include "libnrtype/font-factory.h"
 
 #include "object/sp-flowtext.h"
 #include "object/sp-text.h"
@@ -40,13 +41,6 @@ namespace Inkscape {
 namespace UI {
 namespace Dialog {
 
-
-GlyphsPanel &GlyphsPanel::getInstance()
-{
-    return *new GlyphsPanel();
-}
-
-
 static std::map<GUnicodeScript, Glib::ustring> & getScriptToName()
 {
     static bool init = false;
@@ -593,7 +587,7 @@ void GlyphsPanel::insertText()
     SPItem *textItem = nullptr;
     auto itemlist = selection->items();
         for(auto i=itemlist.begin(); itemlist.end() != i; ++i) {
-            if (SP_IS_TEXT(*i) || SP_IS_FLOWTEXT(*i)) {
+            if (is<SPText>(*i) || is<SPFlowtext>(*i)) {
             textItem = *i;
             break;
         }
@@ -674,7 +668,7 @@ void GlyphsPanel::calcCanInsert()
     int items = 0;
     auto itemlist = selection->items();
     for(auto i=itemlist.begin(); itemlist.end() != i; ++i) {
-        if (SP_IS_TEXT(*i) || SP_IS_FLOWTEXT(*i)) {
+        if (is<SPText>(*i) || is<SPFlowtext>(*i)) {
             ++items;
         }
     }
@@ -710,9 +704,9 @@ void GlyphsPanel::rebuild()
 {
     Glib::ustring fontspec = fontSelector->get_fontspec();
 
-    font_instance* font = nullptr;
-    if( !fontspec.empty() ) {
-        font = font_factory::Default()->FaceFromFontSpecification( fontspec.c_str() );
+    std::shared_ptr<FontInstance> font;
+    if (!fontspec.empty()) {
+        font = FontFactory::get().FaceFromFontSpecification(fontspec.c_str());
     }
 
     if (font) {
